from GGI.ggi import Visual
from GGI.const import flagAsync, emPointer, emKeyPress, emKey, evPtrRelative, evPtrAbsolute, evPtrButtonPress, evPtrButtonRelease
from GGI.exception import Timeout

from random import randint

def clamp(x,a,b):
    if x > b: return b
    if x < a: return a
    return x

def drawbox(v,x1,y1,x2,y2,c):
    v.fg = c
    v.putHLine(x1, y1, x2-x1)
    v.putHLine(x1, y2, x2-x1)
    v.putVLine(x1, y1, y2-y1)
    v.putVLine(x2, y1, y2-y1)
    
class Snake:
    def __init__(self,x,y, x1,y1,x2, y2, length = 10, speed = 10 ):
        self.max = length
        self.body = [ (x, y, x, y, randint(0,0xffffff)) ]
        self.speed = speed
        self.xmin, self.xmax, self.ymin, self.ymax = x1, x2, y1, y2

    def draw(self, v):
        if len(self.body)>self.max:
            i=self.body[0]
            v.putLine(i[0],i[1],i[2],i[3],0)
            del self.body[0]
        for i in self.body:
            v.putLine(i[0],i[1],i[2],i[3],i[4])        

    def evolve(self):
        ex, ey = self.body[-1][2], self.body[-1][3]
        self.body.append((ex,ey,
                          clamp(ex+randint(-self.speed,self.speed),
                                self.xmin, self.xmax),
                          clamp(ey+randint(-self.speed,self.speed),
                                self.ymin, self.ymax),
                          randint(0,0xffffff)))

vis = Visual()
vis.mode = vis.Mode()
size = vis.mode.visible
vis.flags = flagAsync
vis.mask= emPointer | emKeyPress

snakes = [ Snake(30,30, 20, 60, 150, 200, speed=30),
           Snake(200, 200, 0,0, size[0], size[1], length=20) ]

px = size[0]/2
py = size[1]/2
selected = None
box = None
updatebox = 0
while 1:
    try:
        e,t = vis.poll(mask=(emPointer | emKeyPress), timeout=0)
        if e & emKey: break
        ev = vis.read(emPointer)
        while(ev):
            t = ev.type
            if t == evPtrAbsolute:
                if selected: updatebox = 1
                px, py = ev.x, ev.y
            elif t == evPtrRelative:
                if selected: updatebox = 1
                px += ev.x; py += ev.y
            elif t == evPtrButtonPress: selected = (px, py)
            elif t == evPtrButtonRelease and selected:
                snakes.append(Snake(box[0],box[1],
                                    box[0],box[1],box[2],box[3]))
                updatebox = 1
                selected = None
            ev = vis.read(emPointer)
        if updatebox:
            drawbox(vis, box[0], box[1], box[2], box[3], 0)
        updatebox = 0
        if selected:
            box = (min(px,selected[0]),
                   min(py,selected[1]),
                   max(px,selected[0]),
                   max(py,selected[1]))
            drawbox(vis, box[0], box[1], box[2], box[3], 0xffffff)
    except Timeout:
        pass
        
    for i in snakes:
        i.evolve()
        i.draw(vis)
    vis.flush()
